# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2021-05-25 12:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('code', models.CharField(db_index=True, max_length=20)),
                ('municipality_levels', models.CharField(default='', max_length=200)),
                ('data_loaded', models.BooleanField(db_index=True, default=False)),
                ('it_codice_catastale', models.CharField(blank=True, max_length=4, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='GeonamesAdm1',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('code', models.CharField(db_index=True, max_length=20)),
                ('feature_code', models.CharField(max_length=20)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='geonames.Country')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeonamesAdm2',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('code', models.CharField(db_index=True, max_length=20)),
                ('feature_code', models.CharField(max_length=20)),
                ('adm1', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm1')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeonamesAdm3',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('code', models.CharField(db_index=True, max_length=20)),
                ('postal_code', models.CharField(max_length=10, verbose_name='Postal Code')),
                ('it_codice_catastale', models.CharField(blank=True, max_length=4, null=True)),
                ('adm2', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm2')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeonamesAdm4',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('code', models.CharField(db_index=True, max_length=20)),
                ('adm2', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm2')),
                ('adm3', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm3')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeonamesAdm5',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('adm4', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm4')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PopulatedPlace',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('alternate_names', models.CharField(db_index=True, default='', max_length=2000)),
                ('suppressed', models.BooleanField(default=True)),
                ('feature_code', models.CharField(db_index=True, max_length=20)),
                ('adm1', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm1')),
                ('adm2', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm2')),
                ('adm3', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm3')),
                ('adm4', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='geonames.GeonamesAdm4')),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='geonames.Country')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
